/*
    Language:	English
    Platform:	Win9x/NT/7
    Author:	J. Lipman (info@joellipman.com)
    Website:	www.joellipman.com
    Copyright:	Copyright (C) 2010 Joellipman.Com
    License:	GNU/GPL http://www.gnu.org/copyleft/gpl.html

    Program:	Network Transmission Dectector
    Version:	1.0
    Created:	20/09/2011

    Script Function:
	This program has been created to monitor any transimissions going
	in and out of the client workstation it is run from.

    License Details:
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


; *************************************************************************************************************
; USUAL SCRIPT STUFF

	#NoEnv
	SendMode Input
	SetWorkingDir %A_ScriptDir%
	#SingleInstance Force		; Skips the prompt that a current instance is running and overwrites this one with a fresh instance of this program
	SetFormat, float, 0.2       ; Round any floats to two decimals

	; To autostart with windows (optional)
	; RegWrite, REG_SZ, HKEY_CURRENT_USER, Software\Microsoft\Windows\CurrentVersion\Run, %ThisProgramName%, %A_ScriptFullPath%

; *************************************************************************************************************
; DECLARE GLOBAL VARIABLES

	ThisProgramName:="Joes Network Transmission Detector"
	ThisProgramVersion:="1.0"
	ThisProgramWidth:=300
	ThisProgramHeight:=300
	ThisProgramFgColor:="White" ; Program Foreground Color
	ThisProgramBgColor:="Black" ; Program Background Color
	ThisProgramPGColor:="Black" ; Progress Background Color
	ThisProgramLV1Color:="White" ; ListView Foreground Color
	ThisProgramLV2Color:="Black" ; ListView Background Color
	ThisProgramMKColor:="White" ; Marker Color
	ThisProgramBgImage:="jdt_bg_001.png"
	ThisProgramStartTime:=A_Hour ":" A_Min ":" A_Sec
	ThisProgramStartDate:=A_Year "-" A_MM "-" A_DD " " ThisProgramStartTime


	DownloadTotal:=0
	UploadTotal:=0
	ColumnFocus:=0

	SysGet, MonitorWorkArea, MonitorWorkArea
	StartXPos := MonitorWorkAreaRight - ThisProgramWidth - 2
	StartYPos := MonitorWorkAreaBottom - ThisProgramHeight

; *************************************************************************************************************
; SETUP MENUS

	; Tray Menu
	Menu, Tray, Tip, %ThisProgramName% v%ThisProgramVersion%
	Menu, Tray, NoStandard
;	Menu, Tray, Add, Configure..., OpenConfig
;	Menu, Tray, Add
	Menu, Tray, Add, Restart, Restart
	Menu, Tray, Add, Exit, GuiClose

	Gui, Menu

; *************************************************************************************************************
; DEFINE GUI ELEMENTS

	Gui, Font, s8, Verdana,
	Gui, Color, %ThisProgramBgColor%
;	Gui, +OwnDialogs -Resize +MinSize%ThisProgramWidth%x%ThisProgramHeight% +MaxSize4096x2048 -SysMenu +AlwaysOnTop -Caption +0x40000
	Gui, +OwnDialogs -Caption +ToolWindow -SysMenu

	; add the background image if it exists
;	If FileExist(A_ScriptDir . "\" . ThisProgramBgImage)
;	{
;		ThisBgImage:=A_ScriptDir . "\" . ThisProgramBgImage
;		Gui, Add, Picture, x0 y0, %ThisBgImage%
;	}


	Gui, Font, s10 Bold, Verdana,
	Gui, Add, Text, x10 y7 c%ThisProgramFgColor% vThisProgramTitle, %ThisProgramName%
	Gui, Add, Text, x282 y5 w20 h20 gGuiClose c%ThisProgramFgColor%, X

	Gui, Font, s8 Normal, Verdana,
	Gui, Add, Text, x10 y30 cGreen vDownloadsHeading w280, Received
	marker := Chr( 94 )
	Gui, Add, Text, x10 y105 cRed vUploadsHeading w280, Sent

	pOptions=w10 h50 +Vertical +Background%ThisProgramPGColor% -Smooth
	dnOptions=cGreen y45 %pOptions%
	upOptions=cRed y120 %pOptions%

	Loop, 28
	{
		xPos:=(A_Index=1)? "x10" : "x+0"
		Gui, Add, Progress, %xPos% %dnOptions% vDownload%A_Index%
	}
	Loop, 28
	{
		xPos:=(A_Index=1)? "x10" : "x+0"
		Gui, Add, Progress, %xPos% %upOptions% vUpload%A_Index%
	}

	Gui, Font, s7 Normal, Verdana,
	Gui, Add, Text, x10 y95 c%ThisProgramMKColor% vMarker, %marker%
	Gui, Add, ListView, x10 y180 r8 w280 c%ThisProgramLV1Color% +Background%ThisProgramLV2Color% vListMessages -Hdr -E0x200, Time|Message ; -Grid -0x2000

; *************************************************************************************************************
; START TIMERS

	If GetIfTable(tb)
	   GoSub, Refresh

	SetTimer, NetMeter, On, 1000

; *************************************************************************************************************
; HIDE OBJECTS NOT NEEDED FOR THE MOMENT


; *************************************************************************************************************
; GENERATE AND DISPLAY THE CONTENT

	Gui, Show, w%ThisProgramWidth% h%ThisProgramHeight% x%StartXPos% y%StartYPos% vMainScreen, %ThisProgramName% v%ThisProgramVersion%



;    WinSet, TransColor, Orange, %ThisProgramName%
;    WinSet, Transparent, 100, %ThisProgramName%
	WinSet, TransColor, %ThisProgramBgColor%, %ThisProgramName%

	Return

; *************************************************************************************************************
; FUNCTIONS AND PROCEDURES

	OpenConfig:
	Return

	ShowBarInfo:
	Return

	NetMeter:
		ColumnFocus++

		If ColumnFocus >= 29
			ColumnFocus = 1


		dnNew := 0
		upNew := 0

		GetIfTable(tb)

		Loop, % DecodeInteger(&tb)
		{
		   dnNew += DecodeInteger(&tb + 4 + 860 * (A_Index - 1) + 552)      ; Total Incoming Octets
		   upNew += DecodeInteger(&tb + 4 + 860 * (A_Index - 1) + 576)      ; Total Outgoing Octets
		}

		dnRate := Round((dnNew - dnOld) / 1024)
		upRate := Round((upNew - upOld) / 1024)

		GuiControl,, Download%ColumnFocus%, %dnRate%
		GuiControl,, Upload%ColumnFocus%, %upRate%
		GuiControl, Move, Marker, % "x" ColumnFocus*10
		str:=""

		If dnRate>0 OR upRate>0
		{
			thisDate:=A_Year "-" A_MM "-" A_DD " " A_Hour ":" A_Min ":" A_Sec
			thisStr:=Chr( 18 ) " " dnRate "Kb " A_Space Chr( 24 ) " " upRate "Kb"
			If LV_GetCount() > 7
				LV_Delete(1)
			LV_Add( "", thisDate, thisStr )
			LV_ModifyCol()
			DownloadTotal:=(DownloadTotal + dnRate) * 1
			DispDownloadTotal:=AddCommas( DownloadTotal )
			UploadTotal:=(UploadTotal + upRate) * 1
			DispUploadTotal:=AddCommas( UploadTotal )
			GuiControl,, DownloadsHeading, Received (%DispDownloadTotal%Kb) since %ThisProgramStartTime%
			GuiControl,, UploadsHeading, Sent (%DispUploadTotal%Kb) since %ThisProgramStartTime%
		}
		dnOld := dnNew
		upOld := upNew
	Return

	ClearLists:
		Gui, ListView, ListMessages
		LV_Delete()
		ThisCount:=0
		LV_Add("", "Please wait...")
		LV_ModifyCol()
	Return

	AddCommas(val)
	{
	  val:= RegExReplace(val, "(\d)(?=(?:\d{3})+(?:\.|$))", "$1,")
	  Return val
 	}

	GetIfTable(ByRef tb, bOrder = True)
	{
		nSize := 4 + 860 * GetNumberOfInterfaces() + 8
		VarSetCapacity(tb, nSize)
		Return DllCall("iphlpapi\GetIfTable", "Uint", &tb, "UintP", nSize, "int", bOrder)
	}

	GetIfEntry(ByRef tb, idx)
	{
		VarSetCapacity(tb, 860)
		DllCall("ntdll\RtlFillMemoryUlong", "Uint", &tb + 512, "Uint", 4, "Uint", idx)
		Return DllCall("iphlpapi\GetIfEntry", "Uint", &tb)
	}

	GetNumberOfInterfaces()
	{
		DllCall("iphlpapi\GetNumberOfInterfaces", "UintP", nIf)
		Return nIf
	}

	DecodeInteger(ptr)
	{
		Return *ptr | *++ptr << 8 | *++ptr << 16 | *++ptr << 24
	}

	GetChildHWND(ParentHWND, ChildClassNN)
	{
		WinGetPos, ParentX, ParentY,,, ahk_id %ParentHWND%
		if ParentX =
			return  ; Parent window not found (possibly due to DetectHiddenWindows).
		ControlGetPos, ChildX, ChildY,,, %ChildClassNN%, ahk_id %ParentHWND%
		if ChildX =
			return  ; Child window not found, so return a blank value.
		return DllCall("WindowFromPoint", "int", ChildX + ParentX, "int", ChildY + ParentY)
	}

; =============================================================================================================

; =============================================================================================================

; **********************************************************************************************************************************************************************
; MOUSE MOVEMENT (Tooltips)


; **********************************************************************************************************************************************************************
; 3rd-party Function: SAMPLE



; **********************************************************************************************************************************************************************
; END PROGRAM
; **********************************************************************************************************************************************************************

	Restart:
	Refresh:
		Reload

	GuiClose:
	close:
		SetTimer, NetMeter, Off
		Gui, Destroy
		ExitApp


